--****************************************************************************
-- @file      QtWidgets Xtra OOP wrapper
-- @author    Valentin Schmidt
-- @version   0.3
-- @requires  Xtra("QtWidgets")
--****************************************************************************

property _qx

property _dropHandler
property _dropTarget
property _dropMap

property _detacher

----------------------------------------
-- @constructor
----------------------------------------
on new (me)

  -- load xtra
  me._qx = xtra("QtWidgets").new()

  -- autoload all widget/dialog/object classes
  startNum = member("Qt", "qt").memberNum
  endNum = castLib("qt").member.count
  repeat with i = startNum to endNum
    m = member(i, "qt")
    class = m.name
    if class<>"" then me[symbol(class)] = m.script
  end repeat

  -- load optional extensions (named constants)
  me.inherit(me, script("constants", "qt"))
  me.inherit(me, script("events", "qt"))

  me._detacher = script("detacher", "qt").new()

  me._dropMap = [:]

  return me
end

----------------------------------------
-- Utility: Extends instance by prepending obj to ancestor chain
--
-- @param {instance} tInstance - script instance to be extended
-- @param {instance|script} tInheritFromObj - script (instance) used as ancestor
-- @return {bool} success
----------------------------------------
on inherit (me, tInstance, tInheritFromObj)
  if ilk(tInstance)<>#instance then return false
  if ilk(tInheritFromObj)=#script then tInheritFromObj = tInheritFromObj.new()
  if voidP(tInstance[#ancestor]) then tInstance.setProp(#ancestor, tInheritFromObj)
  else me.inherit(tInstance.ancestor, tInheritFromObj)
  return true
end

----------------------------------------
-- Sets drop callback for widget
-- @param {symbol} dropHandler
-- @param {object} [dropTarget=_movie]
----------------------------------------
on registerDropHandler (me, srcWidget, dropHandler, dropTarget)
  me._dropMap.addProp(srcWidget._id, [dropHandler, dropTarget])
  me._qx.setDropCallback(#_detachDrop, me)
end

----------------------------------------
-- Checks if 2 Qt objects are equal
-- @param {instance} obj1
-- @param {instance} obj2
-- @return {bool}
----------------------------------------
on equal (me, obj1, obj2)
  if ilk(obj1)<>#instance OR ilk(obj2)<>#instance then return (obj1=obj2)
  if obj1._class<>obj2._class then return FALSE
  return (obj1._id=obj2._id)
end

----------------------------------------
-- Destructor called by LSW
-- @callback
----------------------------------------
on __destruct (me)
  me._qx = VOID
end

----------------------------------------
-- Used for "detaching" drop action xtra callbacks from original thread,
-- so scripting errors in callbacks don't fail silently
-- @callback
----------------------------------------
on _detachDrop (me, dropData, dropPoint, dropSourceID)
  l = me._dropMap.getProp(dropSourceID)
  if listP(l) then
    args = [dropData, dropPoint, dropSourceID]
    me._detacher.detach(l[1], l[2], args)
  end if
end

----------------------------------------
-- low level QtWidgets Xtra access
----------------------------------------
on callQt (me, handler, arg)
  if voidP(arg) then
    return call(handler, me._qx)
  else 
    return call(handler, me._qx, arg)
  end if
end
